#!/usr/bin/perl

#┌─────────────────────────────────
#│ Quick DB : admin.cgi - 2022/05/28
#│ copyright (c) KentWeb
#│ http://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Quick DB v4.1 PLUS 2022/05/28
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use vars qw(%in %cf);
use lib "./lib";
use CGI::Minimal;
use CGI::Session;
use Digest::SHA::PurePerl qw(sha256_base64);

# 設定ファイル認識
require "./init.cgi";
%cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
%in = parse_form($cgi);

# 認証
require "./lib/login.pl";
auth_login();

# 管理モード
if ($in{add_data}) { add_data(); }
if ($in{edt_data}) { edt_data(); }
if ($in{csv_data}) { csv_data(); }
if ($in{pass_mgr}) { pass_mgr(); }

# メニュー画面
menu_html();

#-----------------------------------------------------------
#  メニュー画面
#-----------------------------------------------------------
sub menu_html {
	header("メニューTOP");
	print <<EOM;

<div class="menu-msg">選択ボタンを押してください。</div>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<table class="form-tbl">
<tr>
	<th>選択</th>
	<th width="280">処理メニュー</th>
</tr><tr>
	<td><input type="submit" name="add_data" value="選択"></td>
	<td>新規データ作成</td>
</tr><tr>
	<td><input type="submit" name="edt_data" value="選択"></td>
	<td>データ管理</td>
</tr><tr>
	<td><input type="submit" name="csv_data" value="選択"></td>
	<td>CSVデータ管理</td>
</tr><tr>
	<td><input type="submit" name="pass_mgr" value="選択"></td>
	<td>パスワード管理</td>
</tr><tr>
	<td><input type="button" value="選択" onclick="window.open('$cf{db_cgi}','_top')"></td>
	<td class="ta-l">検索画面に戻る</td>
</tr><tr>
	<td><input type="submit" name="logoff" value="選択"></td>
	<td>ログアウト</td>
</tr>
</table>
</form>
</div>
</body>
</html>
EOM
	exit;
}


#-----------------------------------------------------------
#  新規データ追加
#-----------------------------------------------------------
sub add_data {
	# 新規追加
	if ($in{job} eq 'new') { new_data(); }

	# 修正のとき
	my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = @_;
	$com =~ s/<br>/\n/g;
	my %cat = (1 => $cat1, 2 => $cat2, 3 => $cat3);

	# 引数定義
	my $mode = $in{edt_data} ? 'edt_data' : 'add_data';
	my $job = $in{job} ? $in{job} . 2 : 'new';

	# 画面表示
	header('登録フォーム');
	back_btn();
	print <<EOM;
<div class="body">
<div class="ttl">■ 登録/修正フォーム</div>
<form action="$cf{admin_cgi}" method="post" enctype="multipart/form-data">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="$mode" value="1">
<input type="hidden" name="job" value="$job">
<input type="hidden" name="no" value="$in{no}">
<table class="form-tbl">
EOM

	foreach my $i (1 .. 3) {
		print qq|<tr><th>$cf{categ}->[$i-1]</th>|;
		print qq|<td><select name="cat$i">\n|;

		foreach my $op (0 .. $#{$cf{"cate$i"}}) {
			if ($cat{$i} == $op) {
				print qq|<option value="$op" selected>$cf{"cate$i"}->[$op]\n|;
			} else {
				print qq|<option value="$op">$cf{"cate$i"}->[$op]\n|;
			}
		}

		print qq|</select></td></tr>\n|;
	}

	print <<EOM;
<tr>
	<th>タイトル</th>
	<td><input type="text" name="title" size="50" class="reg-ttl" value="$ttl"></td>
</tr><tr>
	<th>本文</th>
	<td><textarea name="comment" cols="55" class="reg-com" rows="7">$com</textarea></td>
</tr><tr>
	<th>ファイル<br>アップ</th>
	<td>
EOM
	my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/<>/);
	my %e = (1 => $e1, 2 => $e2, 3 => $e3);
	foreach my $i (1 .. 3) {
		print qq|添付$i : <input type="file" name="upfile$i" size="40">\n|;
		if ($e{$i}) {
			print qq|[<a href="$cf{uplurl}/$no-$i$e{$i}" target="_blank">添付$i</a>]\n|;
			print qq|&nbsp;<input type="checkbox" name="del$i" value="1">削除\n|;
		}
		print "<br>";
	}
	print <<"EOM";
	</td>
</tr>
</table>
<input type="submit" value="送信する">
</form>
</div>
<div class="clear"></div>
</div>
</body>
</html>
EOM
	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  データ管理
#-----------------------------------------------------------
sub edt_data {
	# 修正フォーム
	if ($in{job} eq 'edit' && $in{no}) {

		my @log;
		open(DB,"$cf{datadir}/db.cgi") or error('open err: db.cgi');
		while(<DB>) {
			my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/<>/);

			if ($in{no} == $no) {
				@log = ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3);
				last;
			}
		}
		close(DB);

		add_data(@log);

	# 修正実行
	} elsif ($in{job} eq 'edit2' && $in{no}) {

		# 入力チェック
		check_input();

		# 画像アップ
		my ($e1n,$w1n,$h1n,$e2n,$w2n,$h2n,$e3n,$w3n,$h3n);
		if ($in{upfile1} || $in{upfile2} || $in{upfile3}) {
			($e1n,$w1n,$h1n,$e2n,$w2n,$h2n,$e3n,$w3n,$h3n) = upload($in{no});
		}

		my (@tmp1,@tmp2,@tmp3,@data);
		open(DB,"+< $cf{datadir}/db.cgi") or error('open err: db.cgi');
		while(<DB>) {
			chomp;
			my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/<>/);

			if ($in{no} == $no) {

			# 画像削除のとき
				if ($in{del1}) {
					unlink("$cf{upldir}/$no-1$e1");
					unlink("$cf{upldir}/$no-s-1$e1") if (-f "$cf{upldir}/$no-s-1$e1");
					$e1 = $w1 = $h1 = "";
				}
				if ($in{del2}) {
					unlink("$cf{upldir}/$no-2$e2");
					unlink("$cf{upldir}/$no-s-2$e2") if (-f "$cf{upldir}/$no-s-2$e2");
					$e2 = $w2 = $h2 = "";
				}
				if ($in{del3}) {
					unlink("$cf{upldir}/$no-3$e3");
					unlink("$cf{upldir}/$no-s-3$e3") if (-f "$cf{upldir}/$no-s-3$e3");
					$e3 = $w3 = $h3 = "";
				}
				# 画像差替（拡張子が異なる場合）
				if ($e1n && $e1n ne $e1) {
					unlink("$cf{upldir}/$no-1$e1");
					unlink("$cf{upldir}/$no-s-1$e1") if (-f "$cf{upldir}/$no-s-1$e1");
				}
				if ($e2n && $e2n ne $e2) {
					unlink("$cf{upldir}/$no-2$e2");
					unlink("$cf{upldir}/$no-s-2$e2") if (-f "$cf{upldir}/$no-s-2$e2");
				}
				if ($e3n && $e3n ne $e3) {
					unlink("$cf{upldir}/$no-3$e3");
					unlink("$cf{upldir}/$no-s-3$e3") if (-f "$cf{upldir}/$no-s-3$e3");
				}

				# 画像差替
				if ($e1n) { $e1 = $e1n; $w1 = $w1n; $h1 = $h1n; }
				if ($e2n) { $e2 = $e2n; $w2 = $w2n; $h2 = $h2n; }
				if ($e3n) { $e3 = $e3n; $w3 = $w3n; $h3 = $h3n; }

				$_ = "$no<>$in{title}<>$in{comment}<>$in{cat1}<>$in{cat2}<>$in{cat3}<>$e1<>$w1<>$h1<>$e2<>$w2<>$h2<>$e3<>$w3<>$h3<>";
				($cat1,$cat2,$cat3) = ($in{cat1},$in{cat2},$in{cat3});
			}
			push(@tmp1,$cat1);
			push(@tmp2,$cat2);
			push(@tmp3,$cat3);
			push(@data,"$_\n");
		}

		# ソート
		@data = @data[sort{$tmp1[$a] <=> $tmp1[$b] || $tmp2[$a] <=> $tmp2[$b] || $tmp3[$a] <=> $tmp3[$b]} 0 .. $#tmp1];

		# 更新
		seek(DB, 0, 0);
		print DB @data;
		truncate(DB, tell(DB));
		close(DB);

		# 完了画面
		message('修正を完了しました');

	# 削除
	} elsif ($in{job} eq 'dele' && $in{no}) {

		# 削除情報
		my %del;
		foreach ( $cgi->param('no') ) {
			$del{$_}++;
		}

		my @log;
		open(DB,"+< $cf{datadir}/db.cgi") or error('open err: db.cgi');
		while(<DB>) {
			chomp;
			my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/<>/);

			# 削除
			if (defined($del{$no})) {

				if ($e1) {
					unlink("$cf{upldir}/$no-1$e1");
					unlink("$cf{upldir}/$no-s-1$e1") if (-f "$cf{upldir}/$no-s-1$e1");
				}
				if ($e2) {
					unlink("$cf{upldir}/$no-2$e2");
					unlink("$cf{upldir}/$no-s-2$e2") if (-f "$cf{upldir}/$no-s-2$e2");
				}
				if ($e3) {
					unlink("$cf{upldir}/$no-3$e3");
					unlink("$cf{upldir}/$no-s-3$e3") if (-f "$cf{upldir}/$no-s-3$e3");
				}
				next;
			}
			push(@log,"$_\n");
		}
		seek(DB, 0, 0);
		print DB @log;
		truncate(DB, tell(DB));
		close(DB);
	}

	# 画面表示
	header('データ修正/削除');
	back_btn();
	print <<EOM;
<div class="body">
<div class="ttl">■ データ修正/削除</div>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="edt_data" value="1">
処理：
<select name="job">
<option value="edit">修正
<option value="dele">削除
</select>
<input type="submit" value="送信">
<br>
<table class="form-tbl02">
<tr>
	<th>選択</th>
	<th>分類</th>
	<th>タイトル</th>
	<th>本文</th>
</tr>
EOM

	open(DB,"$cf{datadir}/db.cgi") or error('open err: db.cgi');
	while(<DB>) {
		my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/<>/);

		my $cat = "$cf{cate1}[$cat1] &gt; $cf{cate2}[$cat2] &gt; $cf{cate3}[$cat3]";
		$com = cut_str($com,30);

		print qq|<tr><td align="center"><input type="checkbox" name="no" value="$no"></td>|;
		print qq|<td>$cat</td>|;
		print qq|<td>$ttl</td>|;
		print qq|<td>$com</td></tr>\n|;
	}
	close(DB);

	print <<EOM;
</table>
</form>
</div>
<div class="clear"></div>
</div>
</body>
</html>
EOM
	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  データ作成
#-----------------------------------------------------------
sub new_data {
	# 入力チェック
	check_input();

	# 採番
	open(DAT,"+< $cf{datadir}/num.dat") or error('open err: num.dat');
	my $num = <DAT> + 1;
	seek(DAT, 0, 0);
	print DAT $num;
	truncate(DAT, tell(DAT));
	close(DAT);

	# 画像アップ
	my ($e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3)
			= $in{upfile1} || $in{upfile2} || $in{upfile3} ? upload($num) : ();

	# データ展開
	my (@tmp1,@tmp2,@tmp3,@data);
	open(DB,"+< $cf{datadir}/db.cgi") or error('open err: db.cgi');
	while(<DB>) {
		my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/<>/);

		push(@tmp1,$cat1);
		push(@tmp2,$cat2);
		push(@tmp3,$cat3);
		push(@data,$_);
	}

	# 追加データ
	push(@tmp1,$in{cat1});
	push(@tmp2,$in{cat2});
	push(@tmp3,$in{cat3});
	push(@data,"$num<>$in{title}<>$in{comment}<>$in{cat1}<>$in{cat2}<>$in{cat3}<>$e1<>$w1<>$h1<>$e2<>$w2<>$h2<>$e3<>$w3<>$h3<>\n");

	# ソート
	@data = @data[sort{$tmp1[$a] <=> $tmp1[$b] || $tmp2[$a] <=> $tmp2[$b] || $tmp3[$a] <=> $tmp3[$b]} 0 .. $#tmp1];

	# 更新
	seek(DB, 0, 0);
	print DB @data;
	truncate(DB, tell(DB));
	close(DB);

	# 完了画面
	message('新規追加しました','new_data');
}

#-----------------------------------------------------------
#  CSVデータ管理
#-----------------------------------------------------------
sub csv_data {
	# ダウン
	if ($in{dl}) {
		require "./lib/jacode.pl";
		
		# 改行
		my $br = $in{br} eq 'win' ? "\r\n" : "\n";
		
		my $items = "データID,タイトル,本文,$cf{categ}[0],$cf{categ}[1],$cf{categ}[2],拡張子,横幅(画像),縦幅(画像)";
		jcode::convert(\$items,'sjis','utf8');

		# ダウンロード用ヘッダ
		print "Content-type: application/octet-stream\n";
		print "Content-Disposition: attachment; filename=data.csv\n\n";

		# バイナリ出力 (Windowsサーバ環境)
		binmode(STDOUT);

		print $items,$br;
		
		open(DB,"$cf{datadir}/db.cgi") or error('open err: db.cgi');
		while( my $db = <DB> ) {
			$db =~ s/,/，/g;
			jcode::convert(\$db,'sjis','utf8');
			my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/<>/);

			print qq|$no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3$br|;
		}
		close(DB);

		exit;

	# アップ
	} elsif ($in{ul} && $in{upfile}) {
		require "./lib/jacode.pl";

		# 一時ファイル
		my $upfile = "$cf{tmpdir}/$$.tmp";

		# 一時ファイル書き込み
		open(UP,"+> $upfile") or error("up err: $upfile");
		binmode(UP);
		print UP $in{upfile};
		close(UP);

		open(DAT,"+< $cf{datadir}/num.dat") or error('open err: num.dat');
		my $num = <DAT>;

		# 一時 → 本番
		my $i = 0;
		open(IN,"$upfile") or error("open err: $upfile");
		my $top = <IN>;
		open(OUT,"> $cf{datadir}/db.cgi") or error('write err: db.cgi');
		while( my $db = <IN> ) {
			$db =~ s/[\r\n]//g;
			jcode::convert(\$db,'utf8','sjis');
			my ($no,$ttl,$com,$cat1,$cat2,$cat3,$e1,$w1,$h1,$e2,$w2,$h2,$e3,$w3,$h3) = split(/,/);
			next if ($db eq '');

			# ID番号
			if ($no eq '') { $no = ++$num; }

			$i++;
			print OUT "$no<>$ttl<>$com<>$cat1<>$cat2<>$cat3<>$e1<>$w1<>$h1<>$e2<>$w2<>$h2<>$e3<>$w3<>$h3<>\n";
		}
		close(OUT);
		close(IN);

		seek(DAT, 0, 0);
		print DAT $num;
		truncate(DAT, tell(DAT));
		close(DAT);

		# 一時ファイル削除
		unlink($upfile);

		# 完了画面
		message("$i件のデータをアップしました",'csv_data');
	}

	# 分類名
	my $catnam;
	foreach my $i (1 .. 3) {
		$catnam .= "<li>分類$i : ";
		foreach my $op (0 .. $#{$cf{"cate$i"}}) {
			$catnam .= qq|$cf{"cate$i"}[$op]→$op, |;
		}
		$catnam =~ s/, $//;
	}

	# 画面表示
	header('CSVデータ管理');
	back_btn();
	print <<EOM;
<div class="body">
<div class="ttl">■ CSVデータ管理</div>
<form action="$cf{admin_cgi}" method="post" enctype="multipart/form-data">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="csv_data" value="1">
<ul>
<li>データファイルをCSV形式でアップ/ダウンロードすることができます。
<li>CSVの順番は、「ID番号, タイトル, 本文, 分類1, 分類2, 分類3, 添付拡張子, 添付横幅, 添付縦幅」です。
</l>
<table class="form-tbl">
<tr>
	<th>CSVダウンロード</th>
	<td>
		ダウンロード先のマシン環境に合わせて形式を選択<br>
		<input type="radio" name="br" value="win" checked>Windows形式
		<input type="radio" name="br" value="mac">Mac/UNIX形式<br>
		<input type="submit" name="dl" value="ダウンロード">
	</td>
</tr><tr>
	<th>CSVアップロード</th>
	<td>
		<input type="file" name="upfile" size="40"><br>
		<input type="submit" name="ul" value="アップロード">
	</td>
</tr>
</table>
<p>■ CSVアップロード時の注意点<p>
<ol>
<li>CSVの1行目は項目行として排除されます（2行目以降を反映）。
<li>CSVの中に、「改行」と「コンマ」は絶対に使用しないでください。
<li>添付ファイルの追加、差替、削除はできません。
<li>新規データを追加する場合は、「ID番号」を空白にしてください。
<li>データの削除はできません。
<li>分類1～3は、数字で指定します。init.cgiで指定する小カテゴリを0から数えた数字になります。
	<ul>
	$catnam
	</ul>
</ol>
</p>
</div>
</div>
<div class="clear"></div>
</body>
</html>
EOM
	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  画像アップロード
#-----------------------------------------------------------
sub upload {
	my $no = shift;

	# サムネイル機能
	require './lib/thumb.pl' if ($cf{thumbnail});

	my @ret;
	foreach my $i (1 .. 3) {
		# 拡張子取得
		my $ext;
		if ($cgi->param_filename("upfile$i") =~ /(\.\w+)$/i) {
			$ext = $1;
		} else {
			push(@ret,('','',''));
			next;
		}
		$ext =~ tr/A-Z/a-z/;
		if ($ext eq '.jpeg') { $ext = '.jpg'; }

		# 添付ファイル定義
		my $upfile = "$cf{upldir}/$no-$i$ext";

		# アップロード書き込み
		open(UP,"+> $upfile") or error("up err: $upfile");
		binmode(UP);
		print UP $in{"upfile$i"};
		close(UP);

		# パーミッション付与
		chmod(0666,$upfile);

		# 画像の場合サイズ取得
		my ($flg,$w,$h);
		if ($ext eq ".jpg") { ($w,$h) = j_size($upfile); $flg++; }
		elsif ($ext eq ".gif") { ($w,$h) = g_size($upfile); $flg++; }
		elsif ($ext eq ".png") { ($w,$h) = p_size($upfile); $flg++; }

		# サムネイル作成
		if ($flg && $cf{thumbnail}) {
			($w,$h) = resize($w,$h);
			my $thumb = "$cf{upldir}/$no-s-$i$ext";
			make_thumb($upfile,$thumb,$w,$h);
		}

		# 拡張子, 横幅, 縦幅 の順
		push(@ret,($ext,$w,$h));
	}
	# 結果を返す
	return @ret;
}

#-----------------------------------------------------------
#  JPEGサイズ認識
#-----------------------------------------------------------
sub j_size {
	my $jpg = shift;

	my ($h,$w,$t);
	open(IMG,"$jpg");
	binmode(IMG);
	read(IMG, $t, 2);
	while (1) {
		read(IMG, $t, 4);
		my ($m, $c, $l) = unpack("a a n", $t);

		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(IMG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(IMG, $t, ($l - 2));
		}
	}
	close(IMG);

	return ($w,$h);
}

#-----------------------------------------------------------
#  GIFサイズ認識
#-----------------------------------------------------------
sub g_size {
	my $gif = shift;

	my $data;
	open(IMG,"$gif");
	binmode(IMG);
	sysread(IMG, $data, 10);
	close(IMG);

	if ($data =~ /^GIF/) { $data = substr($data, -4); }
	my $w = unpack("v", substr($data, 0, 2));
	my $h = unpack("v", substr($data, 2, 2));

	return ($w,$h);
}

#-----------------------------------------------------------
#  PNGサイズ認識
#-----------------------------------------------------------
sub p_size {
	my $png = shift;

	my $data;
	open(IMG,"$png");
	binmode(IMG);
	read(IMG, $data, 24);
	close(IMG);

	my $w = unpack("N", substr($data, 16, 20));
	my $h = unpack("N", substr($data, 20, 24));

	return ($w,$h);
}

#-----------------------------------------------------------
#  HTMLヘッダー
#-----------------------------------------------------------
sub header {
	my $ttl = shift;

	print <<EOM;
Content-type: text/html; charset=utf-8

<!doctype html>
<html lang="ja">
<head>
<meta charset="UTF-8">
<meta name="viewport" content="width=device-width,initial-scale=1.0,minimum-scale=1.0,maximum-scale=1.0,user-scalable=no">
<meta http-equiv="content-style-type" content="text/css" />
<script type="text/javascript"> 
var ua = navigator.userAgent;
if(ua.indexOf('iPhone') != -1 || ua.indexOf('Android') != -1){
//iPhone & Android
document.write('<link rel="stylesheet" href="cmn/mobile_admin.css">');
}else{
//PC
document.write('<link rel="stylesheet" href="cmn/admin.css">');
}
</script>
<link href="cmn/admin.css" rel="stylesheet" type="text/css" />
<title>$ttl</title>
</head>
<body>
<div id="head">
	<div id="main-ttl">
		<h1><img src="$cf{cmnurl}/star.png" alt="star" class="icon"> $cf{version} 管理画面<h1>
	</div>
</div>
<div id="menuwrapper"></div>
<div id="body">
EOM
}

#-----------------------------------------------------------
#  バックボタン
#-----------------------------------------------------------
sub back_btn {
	print <<EOM;
<div class="back-btn">
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<input type="submit" value="&lt; メニュー">
</form>
</div>
EOM
}

#-----------------------------------------------------------
#  フォームデコード
#-----------------------------------------------------------
sub parse_form {
	my $cgi = shift;

	my %in;
	foreach ( $cgi->param() ) {
		my $val = $cgi->param($_);

		if (!/^upfile\d$/) {
			# 無効化
			$val =~ s/&/&amp;/g;
			$val =~ s/</&lt;/g;
			$val =~ s/>/&gt;/g;
			$val =~ s/"/&quot;/g;
			$val =~ s/'/&#39;/g;

			# 改行変換
			$val =~ s/\r\n/<br>/g;
			$val =~ s/\n/<br>/g;
			$val =~ s/\r/<br>/g;
		}
		$in{$_} = $val;
	}
	return %in;
}

#-----------------------------------------------------------
#  完了メッセージ
#-----------------------------------------------------------
sub message {
	my ($msg,$mode) = @_;
	
	header($msg);
	print <<EOM;
<div id="ttl">■ 処理完了</div>
<div id="msg">
<p>$msg</p>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="$mode" value="1">
<p><input type="submit" value="当初の画面に戻る"></p>
</form>
</div>
</div>
</body>
</html>
EOM
	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  エラー
#-----------------------------------------------------------
sub error {
	my $msg = shift;

	header("ERROR!");
	print <<EOM;
<div align="center">
<hr width="350">
<h3>ERROR!</h3>
<p style="color:#dd0000">$msg</p>
<hr width="350">
<form>
<input type="button" value="前画面に戻る" onclick="history.back()">
</form>
</div>
</body>
</html>
EOM
	# フッター
	footer();
	exit;
}

#-----------------------------------------------------------
#  入力チェック
#-----------------------------------------------------------
sub check_input {
	my $err;
	if ($in{title} eq '') { $err .= "タイトルが未入力です<br>"; }
	if ($in{comment} eq '') { $err .= "本文が未入力です<br>"; }
	error($err) if ($err);
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<p style="margin-top:0.5em;text-align:center;font-family:Verdana,Helvetica,Arial;font-size:11pt;">
- <a href="https://www.kent-web.com/" target="_top">$cf{version}</a> -<br>
++ <a href="https://www.transformsite.com/" target="_top">Edited by TRANSFORM</a> ++
</p>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  文字カット for UTF-8
#-----------------------------------------------------------
sub cut_str {
	my ($str,$all) = @_;
	$str =~ s/<br>//g;
	
	my $i = 0;
	my ($ret,$flg);
	while ($str =~ /([\x00-\x7f]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/gx) {
		$i++;
		$ret .= $1;
		if ($i >= $all) {
			$flg++;
			last;
		}
	}
	$ret .= '...' if ($flg);
	
	return $ret;
}

