# モジュール宣言/変数初期化
use strict;
my %cf;
#┌─────────────────────────────────
#│ Quick DB : init.cgi - 2022/05/28
#│ copyright (c) kentweb, 1997-2022
#│ https://www.kent-web.com/
#└─────────────────────────────────
$cf{version} = 'Quick DB v4.1';
#┌─────────────────────────────────
#│ [注意事項]
#│ 1. このプログラムはフリーソフトです。このプログラムを使用した
#│    いかなる損害に対して作者は一切の責任を負いません。
#│ 2. 設置に関する質問はサポート掲示板にお願いいたします。
#│    直接メールによる質問は一切お受けいたしておりません。
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Quick DB v4.1 PLUS: init.cgi - 2022/05/28
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────

#===========================================================
# ■ 基本設定
#===========================================================

# 検索システムタイトル
$cf{html_title} = "Easy dish collection - Quick DB v4.1";

# サムネイル画像を作成する（要：Image::Magick）
# → 縮小画像を自動生成し、画像記事の表示速度を軽くする機能
# 0=no 1=yes
$cf{thumbnail} = 0;

# 大カテゴリ（3種固定）
$cf{categ} = ['食種','材料','調理法'];

# 小カテゴリ：食種（指定個数は制限なし）
$cf{cate1} = ['和食','洋食','中華','お菓子','その他'];

# 小カテゴリ：材料（指定個数は制限なし）
$cf{cate2} = ['肉','魚介類','野菜','卵','豆腐','ご飯','パン','めん類','果物','その他'];

# 小カテゴリ：調理法（指定個数は制限なし）
$cf{cate3} = ['焼く','煮る','炒める','揚げる','蒸す','炊く','鍋物','その他'];

# 検索プログラム【URLパス】
$cf{db_cgi} = './quick.cgi';

# 管理プログラム【URLパス】
$cf{admin_cgi} = './admin.cgi';

# データディレクトリ【サーバパス】
$cf{datadir} = './data';

# 共通ディレクトリ【URLパス】
$cf{cmnurl} = './cmn';

# アップロードディレクトリ
# → 順に、サーバパス、URLパス
$cf{upldir} = './upl';
$cf{uplurl} = './upl';

# アイコンディレクトリ【URLパス】
$cf{iconurl} = './cmn/icon';

# テンプレートディレクトリ【サーバパス】
$cf{tmpldir} = './tmpl';

# 一時ディレクトリ【サーバパス】
$cf{tmpdir} = './data/tmp';

# URLの自動リンク (0=no 1=yes)
$cf{auto_link} = 1;

# 検索結果ページ当たり表示件数
$cf{logs} = 10;

# 新着表示件数
$cf{new_logs} = 5;

# 検索画面からの戻り先【URLパス】
$cf{topurl} = '../index.html';

# 投稿受理最大サイズ (bytes)
# → 例 : 102400 = 100KB
$cf{maxdata} = 51200000;

# 画像ファイルの最大表示の大きさ（単位：ピクセル）
# → これを超える画像は縮小表示します
$cf{max_img_w} = 300;	# 横幅
$cf{max_img_h} = 200;	# 縦幅

# 新着表示画像ファイルの最大表示の大きさ（単位：ピクセル）
# → これを超える画像は縮小表示します
$cf{new_max_img_w} = 80;	# 横幅
$cf{new_max_img_h} = 50;	# 縦幅

# アップロードを許可するファイル（0=no 1=yes）
$cf{ok_text}  = 0;  # TEXT
$cf{ok_gif}   = 1;  # GIF
$cf{ok_jpeg}  = 1;  # JPEG
$cf{ok_png}   = 1;  # PNG
$cf{ok_excel} = 1;  # EXCEL
$cf{ok_word}  = 1;  # WORD
$cf{ok_ppt}   = 0;  # POWER POINT
$cf{ok_zip}   = 1;  # ZIP
$cf{ok_pdf}   = 1;  # PDF

# 拡張子アイコン
# → 各アイコンは$cf{cmnurl}内に格納すること
$cf{ext} = {
	pdf => 'pdf.png',
	xls => 'xls.png',
	xlsx => 'xls.png',
	doc => 'doc.png',
	docx => 'doc.png',
	zip => 'zip.png',
	rm  => 'rm.png',
	mov => 'mov.png',
	wmv => 'wmv.png',
	txt => 'txt.png',
	};

# 上記以外の拡張子アイコン
$cf{ext_o} = 'bin.png';

# クッキーID名（特に変更しなくてよい）
# → クッキー保存名
$cf{cookie_id} = "quickdb";

# 管理パスワードの最大間違い制限
# → この回数以上パスワードを間違うとロックします
$cf{max_failpass} = 5;

# 管理パスワードのロック期間：自動解除を日数で指定
# → この値を 0 にすると自動解除しません。
$cf{lock_days} = 14;

#===========================================================
# ■ 設定完了
#===========================================================

# 設定値を返す
sub set_init { return %cf; }

#-----------------------------------------------------------
#  画像リサイズ
#-----------------------------------------------------------
sub resize {
	my ($w,$h) = @_;

	# 画像表示縮小
	if ($w > $cf{max_img_w} || $h > $cf{max_img_h}) {
		my $w2 = $cf{max_img_w} / $w;
		my $h2 = $cf{max_img_h} / $h;
		my $key;
		if ($w2 < $h2) { $key = $w2; } else { $key = $h2; }
		$w = int ($w * $key) || 1;
		$h = int ($h * $key) || 1;
	}
	return ($w,$h);
}

#-----------------------------------------------------------
#  新着表示画像リサイズ
#-----------------------------------------------------------
sub news_resize {
	my ($w,$h) = @_;

	# 画像表示縮小
	if ($w > $cf{new_max_img_w} || $h > $cf{new_max_img_h}) {
		my $w2 = $cf{new_max_img_w} / $w;
		my $h2 = $cf{new_max_img_h} / $h;
		my $key;
		if ($w2 < $h2) { $key = $w2; } else { $key = $h2; }
		$w = int ($w * $key) || 1;
		$h = int ($h * $key) || 1;
	}
	return ($w,$h);
}

1;

